function [ x , lpnorm , ts] = lpDPM_v6( x0,P,thr,M)
% This code was downloaded from radarmimo.com
% % Mohammad Alaee, 26.09.2016
% % mmalaee@yahoo.com

if ~exist('x0','var')
    rng(1)
    x0          = mcode(8,36);
end
if ~exist('M','var')
   M            = 256;
end
if ~exist('thr','var')
    thr         = 1e-5;
end
if ~exist('P','var')
    P           = 1 : 13;
end

N               = length(x0);

% % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
x0              = exp(1i * floor(angle(x0(1:N))/(2*pi/M))*(2*pi/M));

% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
x               = x0;
L               = length(P);
p1              = pslisl(x);
rkln            = abs(xcorr(x));
rkl             = rkln(1:N-1);
t               = norm(rkl,2);
lpnorm(1)       = t;


tic
for l = 1 : L
    cnd         = 1;
    p           = 2^(P(l));
    rkln        = abs(xcorr(x));
    rkl         = rkln(1:N-1);
    t           = norm(rkl,p);
    if t^p == inf
        continue
    end
    
    while cnd
        
        for d = 1 : N
            rkln            = abs(xcorr(x));
            rkl             = rkln(1:N-1);
            t               = norm(rkl,p);
            if d == 1
                if M <= 2
                    [ a_dk,c_dk ] = mabcbin( x,d );
                    X1          = [a_dk,c_dk];
                else
                    [ a_dk,b_dk,c_dk ] = mabc( x,d );
                    X1          = [a_dk,c_dk,b_dk];
                end
            else
                if M <= 2
                    [ a_dk,c_dk ] = mabcbin_v2( x,d ,a_dk,c_dk );
                    X1          = [a_dk,c_dk];
                else
                    [ a_dk,b_dk,c_dk ] = mabc_v2( x,d ,a_dk,b_dk,c_dk);
                    X1          = [a_dk,c_dk,b_dk];
                end
            end
            
            Y               = (fft(X1,M,2)).';
            Y1_1            = abs(Y);
            Y1_2            = Y1_1.^2;
            
            tau_k           = (t^p - Y1_1.^p - p * Y1_1.^(p-1).*(t - Y1_1)) ./ (t - Y1_1).^2;
            lambda_k        = (p * Y1_1.^(p-1) - 2 * tau_k .* Y1_1);
            
            Omega           = real(tau_k .* Y1_2 + lambda_k .* Y1_1 );
            
            mYobj           = (sum(Omega.')).^(1/p);
            [~,iy]          = min(mYobj);
            phf1est         = 2*pi*(iy-1)/M;
            if phf1est > pi
                phf1est     = phf1est - 2*pi;
            end
            x(d)           = exp(1i * phf1est);
        end
        
        rkln                = abs(xcorr(x));
        rkl                 = rkln(1:N-1);
        lpnorm(end+1)       = norm(rkl,p);
        
        if (lpnorm(end-1) - lpnorm(end))/lpnorm(end) < thr/p
            cnd             = 0;
        end
    end
    p1(end+1)           = pslisl(x);
end

ts = toc;
if M == 2
    x = real(x);
end
if nargout == 0
    MarkerSize = 20;

    figure; 
    plot(10*log10((p1)),'kv-');
    xlabel('iteration')
    ylabel('PSL (dB)')
    grid on
    box on
    
    figure; hold on
    
    plot(10*log10(abs(xcorr(x0))/N),'b.-','MarkerSize',MarkerSize,'MarkerIndices',1:5:2*N-1);
    plot(10*log10(abs(xcorr(x))/N),'kv-','MarkerSize',MarkerSize,'MarkerIndices',1:5:2*N-1);
    ylim([-20*log10(2*sqrt(N)),0])
    
    box on
    grid on
    legend('Initial Sequence','Obtained Sequence','location','best')
    xlabel('shift (k)')
    ylabel('Correlation Level (dB)')

    [psl_x,isl_x] = pslisl(x)
    x      = [];
    
end

